/*
   ZoranDBG.c

   ZORAN ZR36120 module driver.
   Copyright (C) 1997  Vincent Labie (try ivsparis@pratique.fr with
				      for Vincent Labie in the suject)

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include	"Zoran.h"
#include	"ZoranIO.h"

int		ZRVerbose	=ZORAN_ZRVERBOSE;
int		I2CVerbose	=ZORAN_I2CVERBOSE;
int		SAAVerbose	=ZORAN_SAAVERBOSE;

static char *	szTextPol[]		={"+","-"};
static char *	szTextEndian[]		={"Big","Little"};
static char *	szTextPack24[]		={"32","24"};
static char *	szTextYUV2RGB[]		={"YUV","RGB888","RGB565","RGB555"};
static char *	szTextDispMod[]		={"Emulated","Single"};
static char *	szTextDupFld[]		={"No","Duplicated"};
static char *	szTextHFilter[]		={"L0-C3","L3-C3","L4-C4","L5-C4","L4-C4","L0-C3","L0-C3","L0-C3"};
static char *	szTextTopField[][2]	={{"Top=HSYNC low","Top=HSYNC high"},{"Top=FI low","Top=FI high"}};
static char *	szTextExtFI[]		={"Int","Ext"};
static char *	szTextSnapShot[]	={"Cont","Grab"};
static char *	szTextTriCom[]		={"No","Sorry"};
static char *	szTextEnable[]		={"Disable","Enable"};
static char *	szTextSoftReset[]	={"Reset","Running"};

/**********************************************************************************************************************/
	int	ZoranDumpRegBuf(ZORAN * pZoran,char * BufferIn,int BufferLenghtIn)
{
	ZR36120 *		pZR		=pZoran->pZR36120;
	char *			Buffer		=BufferIn;
	int			BufferLenght	=BufferLenghtIn;
	int			l;

	l=sprintf(Buffer,"Zoran%d Dump Reg\n",pZoran->IndexPCI);
	Buffer+=l; BufferLenght-=l; if (BufferLenght<128) goto end;
	l=sprintf(Buffer," Bus=%d\n",(int)pZoran->Bus);
	Buffer+=l; BufferLenght-=l; if (BufferLenght<128) goto end;
	l=sprintf(Buffer," dfn=%d\n",(int)pZoran->dfn);
	Buffer+=l; BufferLenght-=l; if (BufferLenght<128) goto end;
	l=sprintf(Buffer," Irq=%d\n",(int)pZoran->Irq);
	Buffer+=l; BufferLenght-=l; if (BufferLenght<128) goto end;
	l=sprintf(Buffer," bOpen=%d\n",pZoran->bOpen);
	Buffer+=l; BufferLenght-=l; if (BufferLenght<128) goto end;
	l=sprintf(Buffer," pFrameBuffer=0x%08X\n",(u32)pZoran->pFrameBuffer);
	Buffer+=l; BufferLenght-=l; if (BufferLenght<128) goto end;
	l=sprintf(Buffer," FrameBufferSize=0x%08X\n",pZoran->FrameBufferSize);
	Buffer+=l; BufferLenght-=l; if (BufferLenght<128) goto end;
	l=sprintf(Buffer," ZRCapMode=%d\n",pZoran->ZRCapMode);
	Buffer+=l; BufferLenght-=l; if (BufferLenght<128) goto end;
	l=sprintf(Buffer," NbSlot=%d\n",pZoran->NbSlot);
	Buffer+=l; BufferLenght-=l; if (BufferLenght<128) goto end;
	l=sprintf(Buffer," CapSlot=%d\n",pZoran->CapSlot);
	Buffer+=l; BufferLenght-=l; if (BufferLenght<128) goto end;
	l=sprintf(Buffer," UserSlot=%d\n",pZoran->UserSlot);
	Buffer+=l; BufferLenght-=l; if (BufferLenght<128) goto end;
	l=sprintf(Buffer," TabSlotFilled=[%d,%d,%d]\n",
		pZoran->TabSlotFilled[0],pZoran->TabSlotFilled[1],pZoran->TabSlotFilled[2]);
	Buffer+=l; BufferLenght-=l; if (BufferLenght<128) goto end;
	l=sprintf(Buffer," CapWidth=%d\n",pZoran->CapWidth);
	Buffer+=l; BufferLenght-=l; if (BufferLenght<128) goto end;
	l=sprintf(Buffer," CapHeight=%d\n",pZoran->CapHeight);
	Buffer+=l; BufferLenght-=l; if (BufferLenght<128) goto end;
	l=sprintf(Buffer," CapPixelBytes=%d\n",pZoran->CapPixelBytes);
	Buffer+=l; BufferLenght-=l; if (BufferLenght<128) goto end;
	l=sprintf(Buffer," CapScanLineSize=%d\n",pZoran->CapScanLineSize);
	Buffer+=l; BufferLenght-=l; if (BufferLenght<128) goto end;
	l=sprintf(Buffer," ScanLineSize=%d\n",pZoran->ScanLineSize);
	Buffer+=l; BufferLenght-=l; if (BufferLenght<128) goto end;
	l=sprintf(Buffer," bCrop=%d\n",pZoran->bCrop);
	Buffer+=l; BufferLenght-=l; if (BufferLenght<128) goto end;
	l=sprintf(Buffer,"pZR36120=0x%08X\n",(u32)pZR);
	Buffer+=l; BufferLenght-=l; if (BufferLenght<128) goto end;

	l=sprintf(Buffer," 0x%04X ",(u32)(&pZR->HVideoControl)-(u32)pZR);
	Buffer+=l; BufferLenght-=l; if (BufferLenght<128) goto end;
	l=sprintf(Buffer,"HVideoControl=0x%08X\n",*(u32*)&pZR->HVideoControl);
	Buffer+=l; BufferLenght-=l; if (BufferLenght<128) goto end;
	l=sprintf(Buffer,"  HEnd=%d\n",pZR->HVideoControl.HEnd);
	Buffer+=l; BufferLenght-=l; if (BufferLenght<128) goto end;
	l=sprintf(Buffer,"  HStart=%d\n",pZR->HVideoControl.HStart);
	Buffer+=l; BufferLenght-=l; if (BufferLenght<128) goto end;
	l=sprintf(Buffer,"  HSPol=%d [%s]\n",pZR->HVideoControl.HSPol,szTextPol[pZR->HVideoControl.HSPol]);
	Buffer+=l; BufferLenght-=l; if (BufferLenght<128) goto end;
	l=sprintf(Buffer,"\n");
	Buffer+=l; BufferLenght-=l; if (BufferLenght<128) goto end;

	l=sprintf(Buffer," 0x%04X ",(u32)(&pZR->VVideoControl)-(u32)pZR);
	Buffer+=l; BufferLenght-=l; if (BufferLenght<128) goto end;
	l=sprintf(Buffer," VVideoControl=0x%08X\n",*(u32*)&pZR->VVideoControl);
	Buffer+=l; BufferLenght-=l; if (BufferLenght<128) goto end;
	l=sprintf(Buffer,"  VEnd=%d\n",pZR->VVideoControl.VEnd);
	Buffer+=l; BufferLenght-=l; if (BufferLenght<128) goto end;
	l=sprintf(Buffer,"  VStart=%d\n",pZR->VVideoControl.VStart);
	Buffer+=l; BufferLenght-=l; if (BufferLenght<128) goto end;
	l=sprintf(Buffer,"  VSPol=%d [%s]\n",pZR->VVideoControl.VSPol,szTextPol[pZR->VVideoControl.VSPol]);
	Buffer+=l; BufferLenght-=l; if (BufferLenght<128) goto end;
	l=sprintf(Buffer,"\n");
	Buffer+=l; BufferLenght-=l; if (BufferLenght<128) goto end;

	l=sprintf(Buffer," 0x%04X ",(u32)(&pZR->OutVideoControl)-(u32)pZR);
	Buffer+=l; BufferLenght-=l; if (BufferLenght<128) goto end;
	l=sprintf(Buffer," OutVideoControl=0x%08X\n",*(u32*)&pZR->OutVideoControl);
	Buffer+=l; BufferLenght-=l; if (BufferLenght<128) goto end;
	l=sprintf(Buffer,"  LittleEndian=%d [%s]\n",pZR->OutVideoControl.LittleEndian,szTextEndian[pZR->OutVideoControl.LittleEndian]);
	Buffer+=l; BufferLenght-=l; if (BufferLenght<128) goto end;
	l=sprintf(Buffer,"  Pack24=%d [%s]\n",pZR->OutVideoControl.Pack24,szTextPack24[pZR->OutVideoControl.Pack24]);
	Buffer+=l; BufferLenght-=l; if (BufferLenght<128) goto end;
	l=sprintf(Buffer,"  ErrDif=%d\n",pZR->OutVideoControl.ErrDif);
	Buffer+=l; BufferLenght-=l; if (BufferLenght<128) goto end;
	l=sprintf(Buffer,"  YUV2RGB=%d [%s]\n",pZR->OutVideoControl.YUV2RGB,szTextYUV2RGB[pZR->OutVideoControl.YUV2RGB]);
	Buffer+=l; BufferLenght-=l; if (BufferLenght<128) goto end;
	l=sprintf(Buffer,"  DispMod=%d [%s]\n",pZR->OutVideoControl.DispMod,szTextDispMod[pZR->OutVideoControl.DispMod]);
	Buffer+=l; BufferLenght-=l; if (BufferLenght<128) goto end;
	l=sprintf(Buffer,"  VerDcm=%d\n",pZR->OutVideoControl.VerDcm);
	Buffer+=l; BufferLenght-=l; if (BufferLenght<128) goto end;
	l=sprintf(Buffer,"  HorDcm=%d\n",pZR->OutVideoControl.HorDcm);
	Buffer+=l; BufferLenght-=l; if (BufferLenght<128) goto end;
	l=sprintf(Buffer,"  DupFld=%d [%s]\n",pZR->OutVideoControl.DupFld,szTextDupFld[pZR->OutVideoControl.DupFld]);
	Buffer+=l; BufferLenght-=l; if (BufferLenght<128) goto end;
	l=sprintf(Buffer,"  HFilter=%d [%s]\n",pZR->OutVideoControl.HFilter,szTextHFilter[pZR->OutVideoControl.HFilter]);
	Buffer+=l; BufferLenght-=l; if (BufferLenght<128) goto end;
	l=sprintf(Buffer,"  VCLKPol=%d [%s]\n",pZR->OutVideoControl.VCLKPol,szTextPol[pZR->OutVideoControl.VCLKPol]);
	Buffer+=l; BufferLenght-=l; if (BufferLenght<128) goto end;
	l=sprintf(Buffer,"  TopField=%d [%s]\n",pZR->OutVideoControl.TopField,szTextTopField[pZR->OutVideoControl.ExtFI][pZR->OutVideoControl.TopField]);
	Buffer+=l; BufferLenght-=l; if (BufferLenght<128) goto end;
	l=sprintf(Buffer,"  ExtFI=%d [%s]\n",pZR->OutVideoControl.ExtFI,szTextExtFI[pZR->OutVideoControl.ExtFI]);
	Buffer+=l; BufferLenght-=l; if (BufferLenght<128) goto end;
	l=sprintf(Buffer,"\n");
	Buffer+=l; BufferLenght-=l; if (BufferLenght<128) goto end;

	l=sprintf(Buffer," 0x%04X ",(u32)(&pZR->AdrTopVideo)-(u32)pZR);
	Buffer+=l; BufferLenght-=l; if (BufferLenght<128) goto end;
	l=sprintf(Buffer," AdrTopVideo=0x%08X\n",*(u32*)&pZR->AdrTopVideo);
	Buffer+=l; BufferLenght-=l; if (BufferLenght<128) goto end;
	l=sprintf(Buffer,"\n");
	Buffer+=l; BufferLenght-=l; if (BufferLenght<128) goto end;

	l=sprintf(Buffer," 0x%04X ",(u32)(&pZR->AdrBottomVideo)-(u32)pZR);
	Buffer+=l; BufferLenght-=l; if (BufferLenght<128) goto end;
	l=sprintf(Buffer," AdrBottomVideo=0x%08X\n",*(u32*)&pZR->AdrBottomVideo);
	Buffer+=l; BufferLenght-=l; if (BufferLenght<128) goto end;
	l=sprintf(Buffer,"\n");
	Buffer+=l; BufferLenght-=l; if (BufferLenght<128) goto end;

	l=sprintf(Buffer," 0x%04X ",(u32)(&pZR->StrideGrabControl)-(u32)pZR);
	Buffer+=l; BufferLenght-=l; if (BufferLenght<128) goto end;
	l=sprintf(Buffer," StrideGrabControl=0x%08X\n",*(u32*)&pZR->StrideGrabControl);
	Buffer+=l; BufferLenght-=l; if (BufferLenght<128) goto end;
	l=sprintf(Buffer,"  FrameGrab=%d\n",pZR->StrideGrabControl.FrameGrab);
	Buffer+=l; BufferLenght-=l; if (BufferLenght<128) goto end;
	l=sprintf(Buffer,"  SnapShot=%d [%s]\n",pZR->StrideGrabControl.SnapShot,szTextSnapShot[pZR->StrideGrabControl.SnapShot]);
	Buffer+=l; BufferLenght-=l; if (BufferLenght<128) goto end;
	l=sprintf(Buffer,"  VidOvf=%d\n",pZR->StrideGrabControl.VidOvf);
	Buffer+=l; BufferLenght-=l; if (BufferLenght<128) goto end;
	l=sprintf(Buffer,"  DispStride=%d\n",pZR->StrideGrabControl.DispStride);
	Buffer+=l; BufferLenght-=l; if (BufferLenght<128) goto end;
	l=sprintf(Buffer,"\n");
	Buffer+=l; BufferLenght-=l; if (BufferLenght<128) goto end;

	l=sprintf(Buffer," 0x%04X ",(u32)(&pZR->VideoControl)-(u32)pZR);
	Buffer+=l; BufferLenght-=l; if (BufferLenght<128) goto end;
	l=sprintf(Buffer," VideoControl=0x%08X\n",*(u32*)&pZR->VideoControl);
	Buffer+=l; BufferLenght-=l; if (BufferLenght<128) goto end;
	l=sprintf(Buffer,"  VidWinWid=%d\n",pZR->VideoControl.VidWinWid);
	Buffer+=l; BufferLenght-=l; if (BufferLenght<128) goto end;
	l=sprintf(Buffer,"  VidWinHt=%d\n",pZR->VideoControl.VidWinHt);
	Buffer+=l; BufferLenght-=l; if (BufferLenght<128) goto end;
	l=sprintf(Buffer,"  TriCom=%d [%s]\n",pZR->VideoControl.TriCom,szTextTriCom[pZR->VideoControl.TriCom]);
	Buffer+=l; BufferLenght-=l; if (BufferLenght<128) goto end;
	l=sprintf(Buffer,"  MinPix=%d\n",pZR->VideoControl.MinPix);
	Buffer+=l; BufferLenght-=l; if (BufferLenght<128) goto end;
	l=sprintf(Buffer,"  VidEn=%d [%s]\n",pZR->VideoControl.VidEn,szTextEnable[pZR->VideoControl.VidEn]);
	Buffer+=l; BufferLenght-=l; if (BufferLenght<128) goto end;
	l=sprintf(Buffer,"\n");
	Buffer+=l; BufferLenght-=l; if (BufferLenght<128) goto end;

	l=sprintf(Buffer," 0x%04X ",(u32)(&pZR->AdrTopVideoMaskMap)-(u32)pZR);
	Buffer+=l; BufferLenght-=l; if (BufferLenght<128) goto end;
	l=sprintf(Buffer," AdrTopVideoMaskMap=0x%08X\n",*(u32*)&pZR->AdrTopVideoMaskMap);
	Buffer+=l; BufferLenght-=l; if (BufferLenght<128) goto end;
	l=sprintf(Buffer,"\n");
	Buffer+=l; BufferLenght-=l; if (BufferLenght<128) goto end;

	l=sprintf(Buffer," 0x%04X ",(u32)(&pZR->AdrBottomVideoMaskMap)-(u32)pZR);
	Buffer+=l; BufferLenght-=l; if (BufferLenght<128) goto end;
	l=sprintf(Buffer," AdrBottomVideoMaskMap=0x%08X\n",*(u32*)&pZR->AdrBottomVideoMaskMap);
	Buffer+=l; BufferLenght-=l; if (BufferLenght<128) goto end;
	l=sprintf(Buffer,"\n");
	Buffer+=l; BufferLenght-=l; if (BufferLenght<128) goto end;

	l=sprintf(Buffer," 0x%04X ",(u32)(&pZR->OverlayControl)-(u32)pZR);
	Buffer+=l; BufferLenght-=l; if (BufferLenght<128) goto end;
	l=sprintf(Buffer," OverlayControl=0x%08X\n",*(u32*)&pZR->OverlayControl);
	Buffer+=l; BufferLenght-=l; if (BufferLenght<128) goto end;
	l=sprintf(Buffer,"\n");
	Buffer+=l; BufferLenght-=l; if (BufferLenght<128) goto end;

	l=sprintf(Buffer," 0x%04X ",(u32)(&pZR->SysPCIControl)-(u32)pZR);
	Buffer+=l; BufferLenght-=l; if (BufferLenght<128) goto end;
	l=sprintf(Buffer," SysPCIControl=0x%08X\n",*(u32*)&pZR->SysPCIControl);
	Buffer+=l; BufferLenght-=l; if (BufferLenght<128) goto end;
	l=sprintf(Buffer,"  GenPurDir=0x%02X\n",pZR->SysPCIControl.GenPurDir);
	Buffer+=l; BufferLenght-=l; if (BufferLenght<128) goto end;
	l=sprintf(Buffer,"  WaitState=%d\n",pZR->SysPCIControl.WaitState);
	Buffer+=l; BufferLenght-=l; if (BufferLenght<128) goto end;
	l=sprintf(Buffer,"  SoftReset=%d [%s]\n",pZR->SysPCIControl.SoftReset,szTextSoftReset[pZR->SysPCIControl.SoftReset]);
	Buffer+=l; BufferLenght-=l; if (BufferLenght<128) goto end;
	l=sprintf(Buffer,"\n");
	Buffer+=l; BufferLenght-=l; if (BufferLenght<128) goto end;

	l=sprintf(Buffer," 0x%04X ",(u32)(&pZR->GenPurpose)-(u32)pZR);
	Buffer+=l; BufferLenght-=l; if (BufferLenght<128) goto end;
	l=sprintf(Buffer," GenPurpose=0x%08X\n",*(u32*)&pZR->GenPurpose);
	Buffer+=l; BufferLenght-=l; if (BufferLenght<128) goto end;
	l=sprintf(Buffer,"  GenPurIO=0x%02X\n",pZR->GenPurpose.GenPurIO);
	Buffer+=l; BufferLenght-=l; if (BufferLenght<128) goto end;
	l=sprintf(Buffer,"\n");
	Buffer+=l; BufferLenght-=l; if (BufferLenght<128) goto end;

	l=sprintf(Buffer," 0x%04X ",(u32)(&pZR->CodeSource)-(u32)pZR);
	Buffer+=l; BufferLenght-=l; if (BufferLenght<128) goto end;
	l=sprintf(Buffer," CodeSource=0x%08X\n",*(u32*)&pZR->CodeSource);
	Buffer+=l; BufferLenght-=l; if (BufferLenght<128) goto end;
	l=sprintf(Buffer,"\n");
	Buffer+=l; BufferLenght-=l; if (BufferLenght<128) goto end;

	l=sprintf(Buffer," 0x%04X ",(u32)(&pZR->CodeTransferControl)-(u32)pZR);
	Buffer+=l; BufferLenght-=l; if (BufferLenght<128) goto end;
	l=sprintf(Buffer," CodeTransferControl=0x%08X\n",*(u32*)&pZR->CodeTransferControl);
	Buffer+=l; BufferLenght-=l; if (BufferLenght<128) goto end;
	l=sprintf(Buffer,"\n");
	Buffer+=l; BufferLenght-=l; if (BufferLenght<128) goto end;

	l=sprintf(Buffer," 0x%04X ",(u32)(&pZR->CodeMemory)-(u32)pZR);
	Buffer+=l; BufferLenght-=l; if (BufferLenght<128) goto end;
	l=sprintf(Buffer," CodeMemory=0x%08X\n",*(u32*)&pZR->CodeMemory);
	Buffer+=l; BufferLenght-=l; if (BufferLenght<128) goto end;
	l=sprintf(Buffer,"\n");
	Buffer+=l; BufferLenght-=l; if (BufferLenght<128) goto end;

	l=sprintf(Buffer," 0x%04X ",(u32)(&pZR->IRQStatus)-(u32)pZR);
	Buffer+=l; BufferLenght-=l; if (BufferLenght<128) goto end;
	l=sprintf(Buffer," IRQStatus=0x%08X\n",*(u32*)&pZR->IRQStatus);
	Buffer+=l; BufferLenght-=l; if (BufferLenght<128) goto end;
	l=sprintf(Buffer,"  CodRepIRQ=%d\n",pZR->IRQStatus.CodRepIRQ);
	Buffer+=l; BufferLenght-=l; if (BufferLenght<128) goto end;
	l=sprintf(Buffer,"  GIRQ0=%d\n",pZR->IRQStatus.GIRQ0);
	Buffer+=l; BufferLenght-=l; if (BufferLenght<128) goto end;
	l=sprintf(Buffer,"  GIRQ1=%d\n",pZR->IRQStatus.GIRQ1);
	Buffer+=l; BufferLenght-=l; if (BufferLenght<128) goto end;
	l=sprintf(Buffer,"\n");
	Buffer+=l; BufferLenght-=l; if (BufferLenght<128) goto end;

	l=sprintf(Buffer," 0x%04X ",(u32)(&pZR->IRQControl)-(u32)pZR);
	Buffer+=l; BufferLenght-=l; if (BufferLenght<128) goto end;
	l=sprintf(Buffer," IRQControl=0x%08X\n",*(u32*)&pZR->IRQControl);
	Buffer+=l; BufferLenght-=l; if (BufferLenght<128) goto end;
	l=sprintf(Buffer,"  InPinEn=%d\n",pZR->IRQControl.InPinEn);
	Buffer+=l; BufferLenght-=l; if (BufferLenght<128) goto end;
	l=sprintf(Buffer,"  CodRepIRQEn=%d\n",pZR->IRQControl.CodRepIRQEn);
	Buffer+=l; BufferLenght-=l; if (BufferLenght<128) goto end;
	l=sprintf(Buffer,"  GIRQ0En=%d\n",pZR->IRQControl.GIRQ0En);
	Buffer+=l; BufferLenght-=l; if (BufferLenght<128) goto end;
	l=sprintf(Buffer,"  GIRQ1En=%d\n",pZR->IRQControl.GIRQ1En);
	Buffer+=l; BufferLenght-=l; if (BufferLenght<128) goto end;
	l=sprintf(Buffer,"\n");
	Buffer+=l; BufferLenght-=l; if (BufferLenght<128) goto end;

	l=sprintf(Buffer," 0x%04X ",(u32)(&pZR->I2CControl)-(u32)pZR);
	Buffer+=l; BufferLenght-=l; if (BufferLenght<128) goto end;
	l=sprintf(Buffer," I2CControl=0x%08X\n",*(u32*)&pZR->I2CControl);
	Buffer+=l; BufferLenght-=l; if (BufferLenght<128) goto end;
	l=sprintf(Buffer,"  SCL=%d\n",pZR->I2CControl.SCL);
	Buffer+=l; BufferLenght-=l; if (BufferLenght<128) goto end;
	l=sprintf(Buffer,"  SDA=%d\n",pZR->I2CControl.SDA);
	Buffer+=l; BufferLenght-=l; if (BufferLenght<128) goto end;
	l=sprintf(Buffer,"\n");
	Buffer+=l; BufferLenght-=l; if (BufferLenght<128) goto end;

end:	*Buffer=0;
	return(Buffer-BufferIn);
}

/**********************************************************************************************************************/
static char		Buffer[PAGE_SIZE];	// put this buffer on the stack seem to make headhash to the kernel...
	void	ZoranDumpReg(ZORAN * pZoran)
{
	int			l;
	l=ZoranDumpRegBuf(pZoran,Buffer,PAGE_SIZE);
	ERR((Buffer));
	ZRDBG(("ZoranDumpReg size=%i\n",l));
}
